import base64
import time

from dhuolib.config import logger
from dhuolib.services import ServiceAPIML
from dhuolib.utils import validade_name


class DhuolibPlatformClient:
    def __init__(self, service_endpoint=None, project_name=None):
        if not service_endpoint:
            raise ValueError("service_endpoint is required")

        self.service = ServiceAPIML(service_endpoint)
        self.project_name = project_name

    def create_batch_project(self, project_name: str):
        self.project_name = validade_name(project_name)
        response = self.service.create_project(project_name)
        if response.status_code == 400:
            raise ValueError("Project already exists")
        elif response.status_code == 404:
            return ConnectionError("Connection error")

        return response.json()

    def deploy_batch_project(self, script_filename: str, requirements_filename: str):
        if self.project_name is None:
            raise ValueError("Batch project is required")

        if script_filename is None or requirements_filename is None:
            raise ValueError("script_filename and requirements_filename are required")

        try:
            with open(script_filename, "rb") as script_file, open(
                requirements_filename, "rb"
            ) as requirements_file:
                encoded_script = base64.b64encode(script_file.read())
                encoded_requirements = base64.b64encode(requirements_file.read())
                response = self.service.deploy_script(
                    project_name=self.project_name,
                    script_file_encode=encoded_script,
                    requirements_file_enconde=encoded_requirements,
                )
                return response
        except FileNotFoundError as e:
            logger.error(f"Error: {e}")
            return {"error": str(e)}

    def pipeline_status_report(self):
        lst = []
        if self.project_name is None:
            raise ValueError("Batch project is required")
        response = self.service.get_pipeline_status(self.project_name)
        for data in response["data"]:
            lst.append(
                {
                    "date_log": data["date_log"],
                    "step": data["step"],
                    "status": data["status"],
                }
            )
        return lst

    def create_cluster(self, cluster_size: int):
        if self.project_name is None:
            raise ValueError("Batch project is required")

        if cluster_size not in [1, 2, 3]:
            raise ValueError("cluster_size must be 1, 2 or 3")

        response = self.service.create_cluster(self.project_name, cluster_size)
        return response

    def batch_run(self):
        if self.project_name is None:
            raise ValueError("Batch project is required")

        response = self.service.run_pipeline(self.project_name)
        return response



import base64
import sys
import unittest
from unittest.mock import mock_open, patch

from dhuolib.clients.experiment import DhuolibExperimentClient
from dhuolib.config import logger

sys.path.append("src")


class TestDhuolibPlatformClient(unittest.TestCase):
    def setUp(self):
        self.end_point = "http://localhost:8000"
        self.dhuolib = DhuolibExperimentClient(service_endpoint=self.end_point)

    def test_0_invalid_run_params(self):
        response = self.dhuolib.create_experiment(experiment_name="teste1")
        self.assertEqual(list(response.keys()), ["error"])

    @patch("requests.post")
    def test_1_create_experiment_with_valid_params(self, mock_post):
        mock_response = mock_post.return_value
        mock_response.status_code = 201
        mock_response.json.return_value = {"experiment_id": "1"}

        response = self.dhuolib.create_experiment(
            experiment_name="teste1",
            experiment_tags={"version": "v1", "priority": "P1"},
        )
        self.assertEqual(response, mock_response.json.return_value)

    @patch("requests.post")
    def test_2_run_experiment_with_valid_params(self, mock_post):
        mock_response = mock_post.return_value
        mock_response.status_code = 201
        mock_response.json.return_value = {
            "experiment_id": "experiment_id",
            "run_id": "run_id",
            "model_uri": "model_uri",
        }

        response = self.dhuolib.run_experiment(
            type_model="teste1",
            experiment_id="2",
            modelpkl_path="tests/files/LogisticRegression_best.pickle",
            requirements_path="tests/files/requirements.txt",
        )

        self.assertEqual(response, mock_response.json.return_value)

    @patch("requests.post")
    def test_3_create_model_with_valid_params(self, mock_post):
        mock_response = mock_post.return_value
        mock_response.status_code = 201
        mock_response.json.return_value = {
            "current_stage": "Production",
            "last_updated_timestamp": 1713582060414,
            "model_version": "1",
            "run_id": "9434e517ed104958b6f5f47d33c79184",
            "status": "READY",
        }

        run_params = {
            "stage": "Production",
            "modelname": "nlp_framework",
            "modeltag": "v1",
            "run_id": "9434e517ed104958b6f5f47d33c79184",
            "requirements_file": "tests/files/requirements.txt",
            "model_uri": "model_uri",
        }

        response = self.dhuolib.create_model(run_params)

        self.assertEqual(response, mock_response.json.return_value)

    @patch("requests.post")
    def test_4_predict_online_with_valid_dataset(self, mock_post):
        mock_response = mock_post.return_value
        mock_response.status_code = 201
        mock_response.json.return_value = {
            "model_name": "nlp_framework",
            "predictions": [0, 1, 0, 1, 0, 1, 0, 1, 0, 1],
        }

        run_params = {
            "stage": "Production",
            "data": "tests/files/data_predict.csv",
            "modelname": "nlp_framework",
        }

        response = self.dhuolib.predict_online(run_params)

        self.assertEqual(response, mock_response.json.return_value)



import unittest
from sqlalchemy import create_engine, MetaData, Table, Column, Integer, String
from sqlalchemy.orm import sessionmaker

# Adjust this import according to your project structure
from dhuolib.repository import DatabaseConnection, GenericRepository
from sqlalchemy import text
import pandas as pd


class TestDatabaseConnection(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.config_file_name = "tests/files/database.json"
        cls.db_connection = DatabaseConnection(config_file_name=cls.config_file_name)

    def test_database_connection_init(self):
        db_connection = DatabaseConnection(self.config_file_name)
        self.assertIsNotNone(db_connection.engine)
        self.assertIsNotNone(db_connection.session)

    def test_session_scope(self):
        with self.db_connection.session_scope() as session:
            self.assertIsNotNone(session)


class TestGenericRepository(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.config_file_name = "tests/files/database.json"
        cls.db_connection = DatabaseConnection(config_file_name=cls.config_file_name)
        cls.table_name = "test_table"
        cls.repo = GenericRepository(db_connection=cls.db_connection)
        cls.repo.db.session.execute(
            text(
                f"CREATE TABLE IF NOT EXISTS {cls.table_name} (id INTEGER PRIMARY KEY, name VARCHAR(255), age INTEGER, predict VARCHAR(255), version VARCHAR(255), created_at TIMESTAMP)"
            )
        )

    @classmethod
    def tearDownClass(cls):
        with cls.db_connection.session_scope() as session:
            session.execute(text(f"DROP TABLE IF EXISTS {cls.table_name}"))

    def setUp(self):
        self.db_connection = self.__class__.db_connection
        self.repo = self.__class__.repo
        self.table_name = self.__class__.table_name

    def tearDown(self):
        with self.db_connection.session_scope() as session:
            session.execute(text(f"DELETE FROM {self.table_name}"))

    def test_1_insert(self):
        data = {"id": 1, "name": "John Doe", "age": 30}
        inserted = self.repo.insert(self.table_name, data)
        with self.repo.db.session_scope() as session:
            result = session.execute(
                text(f"SELECT * FROM {self.table_name} WHERE id=1")
            ).fetchone()
            self.assertIsNotNone(result)
            self.assertEqual(result.name, "John Doe")
            self.assertEqual(result.age, 30)
            self.assertEqual(inserted.name, "John Doe")
            self.assertEqual(inserted.age, 30)

    def test_2_get_items_with_pagination(self):
        for i in range(10):
            data = {"id": i + 1, "name": f"User {i+1}", "age": 20 + i}
            self.repo.insert(self.table_name, data)

        items = self.repo.get_items_with_pagination(self.table_name, 1, 5)
        self.assertEqual(len(items["items"]), 5)
        self.assertEqual(items["items"][0].name, "User 1")
        self.assertEqual(items["items"][4].name, "User 5")

        items = self.repo.get_items_with_pagination(self.table_name, 2, 5)
        self.assertEqual(len(items["items"]), 5)
        self.assertEqual(items["items"][0].name, "User 6")
        self.assertEqual(items["items"][4].name, "User 10")

    def test_3_update(self):
        data = {"id": 1, "name": "John Doe", "age": 30, "predict": "Negative"}
        self.repo.insert(table_name=self.table_name, data=data)

        updated_data = {"id": 1, "name": "John Doe", "age": 30, "predict": "Positive"}
        self.repo.update(
            table_name=self.table_name,
            index=updated_data["id"],
            predict=updated_data["predict"],
        )

        with self.repo.db.session_scope() as session:
            result = session.execute(
                text(
                    f"SELECT * FROM {self.table_name} WHERE id={int(updated_data['id'])}"
                )
            ).fetchone()
            self.assertIsNotNone(result)
            self.assertEqual(result.predict, "Positive")
            self.assertEqual(result.name, "John Doe")

    def test_4_insert_invalid_data(self):
        invalid_data = {"id": 1, "invalid_column": "Invalid Data"}
        with self.assertRaises(Exception):
            self.repo.insert(self.table_name, invalid_data)

    def test_5_update_table_by_dataframe(self):
        initial_data = pd.DataFrame(
            [{"id": 1, "name": "John Doe", "age": 30, "predict": "Positive"}]
        )
        self.repo.update_table_by_dataframe(
            table_name=self.table_name, df_predict=initial_data, if_exists="replace"
        )

        new_data = pd.DataFrame(
            [
                {
                    "id": 2,
                    "name": "Jane Doe",
                    "age": 25,
                    "predict": "Positive",
                    "version": 1,
                    "created_at": "2021-01-01",
                },
                {
                    "id": 3,
                    "name": "Jim Beam",
                    "age": 35,
                    "predict": "Negative",
                    "version": 1,
                    "created_at": "2021-01-01",
                },
            ]
        )
        self.repo.update_table_by_dataframe(self.table_name, new_data)

        df = self.repo.to_dataframe(self.table_name)
        self.assertEqual(len(df), 3)
        self.assertListEqual(list(df["name"]), ["John Doe", "Jane Doe", "Jim Beam"])
        self.assertListEqual(list(df["predict"]), ["Positive", "Positive", "Negative"])

    def test_6_create_table_by_dataframe(self):
        with self.db_connection.session_scope() as session:
            session.execute(text(f"DROP TABLE IF EXISTS {self.table_name}"))
        data = pd.DataFrame(
            [
                {"id": 1, "name": "John Doe", "age": 30, "predict": "Positive"},
                {"id": 2, "name": "Jane Smith", "age": 25, "predict": "Negative"},
                {"id": 3, "name": "Bob Johnson", "age": 35, "predict": "Positive"},
            ]
        )
        self.repo.create_table_by_dataframe(self.table_name, data)
        with self.repo.db.session_scope() as session:
            result = session.execute(
                text(f"SELECT * FROM {self.table_name}")
            ).fetchall()
            self.assertEqual(len(result), 3)
            self.assertEqual(result[0].name, "John Doe")
            self.assertEqual(result[1].name, "Jane Smith")
            self.assertEqual(result[2].name, "Bob Johnson")